{% extends "blog.twig" %}

{% block blogcontent %}

    <article>
        <h1>{{ title }}</h1>

        <p class="article-information">
            Posted by <a href="https://mnapoli.fr">{{ author }}</a> on {{ date }}
        </p>

        {{ content|raw }}
    </article>

    <h3>
        Comments
    </h3>
    <div id="disqus_thread"></div>
    <noscript>
        Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript">comments powered by Disqus.</a>
    </noscript>

{% endblock %}


{% block javascripts %}
    {{ parent() }}

    <script>
        // Disqus comments
        var disqus_shortname = 'phpdi';

        (function() {
            var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
            dsq.src = '//' + disqus_shortname + '.disqus.com/embed.js';
            (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
        })();
    </script>
{% endblock %}
