<?php

// This is a separated file so that StyleCI doesn't detect it as PHP and doesn't try to reformat it

return [
    'factory' => function () { return 'foo'; }, 'factory2' => function () { return 'bar'; },
];
