<div id="sponsors" class="container">

    <h2><a href="#sponsors">Sponsors</a></h2>

    <p>
        PHP-DI is sponsored by these companies:
    </p>

    <div>
        <a href="https://www.jetbrains.com/" title="JetBrains - Creator of PhpStorm">
            <img src="{{ baseUrl }}/img/sponsors/logo-jetbrains.svg" alt="JetBrains" style="height: 80px; margin-left: 30px"></a>
        <a class="block self-center mx-6 mb-6" href="https://null.tc/" title="Null - Making serverless simpler">
            <img src="{{ baseUrl }}/img/sponsors/logo-null.png" alt="Null" style="width: 70px; margin-left: 50px"></a>
        <a class="block self-center mx-6 mb-6" href="https://www.amezmo.com/" title="Amezmo - Cloud hosting and deployment for PHP">
            <img src="{{ baseUrl }}/img/sponsors/logo-amezmo.svg" alt="Amezmo" style="width: 90px; margin-top: 8px; margin-left: 50px"></a>
        <a class="block self-center mx-6 mb-6" href="https://www.web-id.fr/" title="Web-ID - Laravel web agency">
            <img src="{{ baseUrl }}/img/sponsors/logo-web-id.png" alt="Web-ID" style="height: 45px; margin-left: 50px"></a>
        <a class="block self-center mx-6 mb-6" href="https://activecollab.com/" title="ActiveCollab">
            <img src="{{ baseUrl }}/img/sponsors/logo-activecollab.png" alt="ActiveCollab" style="height: 35px; margin-left: 50px"></a>
    </div>

    <p class="text-center">
        <a href="https://github.com/sponsors/mnapoli" class="btn btn-default">Become a sponsor</a>
    </p>

</div>
