{% extends "default.twig" %}

{% block content %}
    <div class="home">

        <section class="jumbotron">
            <div class="container">
                <h1>
                    PHP-DI <strong>7</strong>
                </h1>
                <p>
                    The dependency injection container for humans
                </p>
                <p>
                    <a href="{{ baseUrl }}/doc/getting-started.html" class="btn btn-lg btn-primary" title="Get started with dependency injection and PHP-DI">
                        Get started
                    </a>
                    <a href="{{ baseUrl }}/doc/" class="btn btn-lg btn-default" title="Read the PHP-DI documentation">
                        Documentation
                    </a>
                </p>
            </div>
        </section>

        <section class="odd">
            {% include 'home_autowiring.twig' %}
        </section>

        <section class="">
            {% include 'home_configuration.twig' %}
        </section>

        <section class="odd">
            {% include 'home_modules.twig' %}
        </section>

        <section class="">
            {% include 'home_container.twig' %}
        </section>

        <section class="odd">
            {% include 'home_frameworks.twig' %}
        </section>

        <section class="">
            {% include 'home_sponsors.twig' %}
        </section>

    </div>

{% endblock %}
