<?php

declare(strict_types=1);

namespace DI\Definition\Exception;

/**
 * Error in the definitions using PHP attributes.
 *
 * @author Matthieu Napoli <matthieu@mnapoli.fr>
 */
class InvalidAttribute extends InvalidDefinition
{
}
