.article-extract {
    border-bottom: solid 1px @border-color;
    margin-bottom: 20px;
    padding: 0 0 20px 0;
}

.article-information {
    font-style: italic;
    font-size: 80%;
    text-align: right;
}

.all-posts li {
    margin-bottom: 10px;
}
