{% extends "default.twig" %}

{% block content %}

    <section class="container">
        <div class="row">
            <div class="col-md-9">

                {% block blogcontent %}

                    <article>

                        <h1>Latest news</h1>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 7.0 released
                            </h3>

                            <p>
                                I am very happy to announce that PHP-DI 7.0 has been released.
                            </p>
                            <a href="{{ baseUrl }}/news/23-php-di-7-0-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 6.0 released
                            </h3>

                            <p>
                                I am very happy to announce that PHP-DI 6.0 has been released.
                            </p>
                            <a href="{{ baseUrl }}/news/22-php-di-6-0-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 6: turning into a compiled container for maximum performances
                            </h3>

                            <p>
                                PHP-DI's motto is "The dependency injection container for humans".
                            </p>
                            <p>
                                This is very important because it defines what PHP-DI tries to be: practical to use, consistent, predictable, with a configuration format that is easy to read and write and a great documentation.
                            </p>
                            <a href="{{ baseUrl }}/news/21-php-di-6-compiled-container.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 5.4 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 5.4 has been released. Here is the list of changes in this new version.
                            </p>
                            <a href="{{ baseUrl }}/news/20-php-di-5-4-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 5.3 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 5.3 has been released. The changes in this new version are:
                            </p>
                            <a href="{{ baseUrl }}/news/19-php-di-5-3-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                Introducing the PHP-DI bridge for Slim
                            </h3>

                            <p>
                                Slim 3 was released 3 months ago and it was significant. It is one of the first frameworks to integrate the latest standards and concepts in its core:
                            </p>
                            <a href="{{ baseUrl }}/news/18-slim-bridge-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 5.2 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 5.2 has been released. This new version comes with:
                            </p>
                            <a href="{{ baseUrl }}/news/17-php-di-5-2-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 5.1 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 5.1 has been released. This new version comes with:
                            </p>
                            <a href="{{ baseUrl }}/news/16-php-di-5-1-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 5.0 released
                            </h3>

                            <p>
                                I am very excited to announce that, after 8 months of work, PHP-DI 5.0 is released.
                            </p>
                            <p>
                                Note: If you have never heard of PHP-DI before, I recommend you to visit <a href="{{ baseUrl }}/">the home page</a> to get an overview of what PHP-DI can do for you.
                            </p>
                            <a href="{{ baseUrl }}/news/15-php-di-5-0-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                Syntactic sugar in PHP-DI 5
                            </h3>

                            <p>
                                It's been quite some time since there was a bit of news here. The reason is that a lot of work has been done for PHP-DI 5! It is not yet ready, but it is coming along great!
                            </p>
                            <p>
                                I am not going to list everything that will be included in this next major version today. For now I'll simply give you a glimpse of the syntactic sugar that it will bring.
                            </p>
                            <a href="{{ baseUrl }}/news/14-php-di-5-syntaxic-sugar.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 4.4 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 4.4 has been released.
                                This version contains mainly improvements to the <code>Container::call()</code> feature, and contains no BC-break.
                            </p>
                            <a href="{{ baseUrl }}/news/13-php-di-4-4-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP 5.6 is out and PHP-DI just got better
                            </h3>

                            <p>
                                PHP 5.6 has been released a month ago and PHP-DI just got better because of it!
                            </p>
                            <p>
                                As explained 9 months ago when I introduced the new definition format for PHP-DI 4,
                                that format has been thought for the future.
                            </p>
                            <a href="{{ baseUrl }}/news/12-php-di-is-better-with-php-5-6.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 4.3 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 4.3 has been released.
                                This version contains features that have all been
                                implemented by contributors other than me, this is very encouraging :)
                                This is also the quickest release, since 4.2 was released only 14 days ago.
                            </p>
                            <p>
                                It comes without any BC break, and with a new main feature: <code>DI\env()</code>
                                to import <strong>environment variables</strong>.
                            </p>
                            <a href="{{ baseUrl }}/news/11-php-di-4-3-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 4.2 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 4.2 has been released.
                            </p>
                            <a href="{{ baseUrl }}/news/10-php-di-4-2-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 4.1 released
                            </h3>

                            <p>
                                I am happy to announce that PHP-DI version 4.1 has been released.
                                This version was focused on stability and documentation improvements,
                                with full backward compatibility.
                            </p>
                            <a href="{{ baseUrl }}/news/09-php-di-4-1-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                10.000 installs!
                            </h3>

                            <p>
                                PHP-DI just reached 200 stars on GitHub and 10 000 installs through
                                <a href="https://packagist.org/packages/mnapoli/php-di">Packagist</a>!
                            </p>
                            <a href="{{ baseUrl }}/news/08-10000-installs.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                PHP-DI 4.0 released
                            </h3>

                            <p>
                                I am extremely happy to announce that, after 4 months of work and more than 150 commits,
                                PHP-DI version 4.0 has been released!
                                This is a new major release, and it delivers. As indicated by "major release",
                                it comes with BC breaks, but this is for the better.
                            </p>
                            <a href="{{ baseUrl }}/news/07-php-di-4-0-released.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                        <div class="article-extract">
                            <h3>
                                What will change in PHP-DI 4: the new definition format
                            </h3>

                            <p>
                                In PHP-DI 4, the definition format will change.
                                Autowiring and Annotations are still there, but the YAML
                                and PHP array definitions have disappeared. Let's see why.
                            </p>
                            <a href="{{ baseUrl }}/news/06-php-di-4-0-new-definitions.html" class="btn btn-default">
                                Read More &gt;</a>
                        </div>

                    </article>

                {% endblock %}

            </div>

            <div class="col-md-3">
                <h3>
                    All posts
                </h3>

                <ul class="all-posts list-unstyled">
                    <li>
                        <a href="{{ baseUrl }}/news/23-php-di-7-0-released.html">PHP-DI 7.0 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/22-php-di-6-0-released.html">PHP-DI 6.0 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/21-php-di-6-compiled-container.html">PHP-DI 6: turning into a compiled container for maximum performances</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/20-php-di-5-4-released.html">PHP-DI 5.4 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/19-php-di-5-3-released.html">PHP-DI 5.3 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/18-slim-bridge-released.html">Introducing the PHP-DI bridge for Slim</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/17-php-di-5-2-released.html">PHP-DI 5.2 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/16-php-di-5-1-released.html">PHP-DI 5.1 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/15-php-di-5-0-released.html">PHP-DI 5.0 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/14-php-di-5-syntaxic-sugar.html">Syntactic sugar in PHP-DI 5</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/13-php-di-4-4-released.html">PHP-DI 4.4 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/12-php-di-is-better-with-php-5-6.html">PHP 5.6 is out and PHP-DI just got better</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/11-php-di-4-3-released.html">PHP-DI 4.3 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/10-php-di-4-2-released.html">PHP-DI 4.2 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/09-php-di-4-1-released.html">PHP-DI 4.1 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/08-10000-installs.html">10.000 installs!</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/07-php-di-4-0-released.html">PHP-DI 4.0 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/06-php-di-4-0-new-definitions.html">What will change in PHP-DI 4: the new definition format</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/05-php-di-3-5.html">PHP-DI 3.5 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/04-php-di-3-4.html">PHP-DI 3.4 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/03-php-di-3-3.html">PHP-DI 3.3 released</a>
                    </li>
                    <li>
                        <a href="{{ baseUrl }}/news/02-php-di-3-2.html">PHP-DI 3.2 released</a>
                    </li>
                </ul>
            </div>

        </div>
    </section>

{% endblock %}
