@import url(https://fonts.googleapis.com/css?family=Source+Sans+Pro:200,400,700,200italic,400italic,700italic);
@import "../bower_components/bootstrap/less/bootstrap.less";
@import "../bower_components/fontawesome/less/font-awesome.less";
@import "variables.less";
@import "home.less";
@import "blog.less";
@import "algolia.less";

body {
    font-family: 'Source Sans Pro', sans-serif;
    -webkit-font-smoothing: antialiased;
    margin-top: 60px;
    font-size: 14px;
    @media (min-width: @screen-sm-min) {
        font-size: 15px;
    }
    @media (min-width: @screen-md-min) {
        font-size: 16px;
    }
}

p {
    line-height: 1.5;
    margin: 10px 0 20px;
}
h1 {
    font-weight: 200;
    font-size: 34px;
    @media (min-width: @screen-sm-min) {
        font-size: 42px;
    }
    @media (min-width: @screen-md-min) {
        font-size: 48px;
    }
}
h2 {
    font-weight: 400;
    color: #525252;
    text-decoration: none;
    margin: 30px 0;
    font-size: 30px;
    @media (min-width: @screen-sm-min) {
        font-size: 33px;
    }
    @media (min-width: @screen-md-min) {
        font-size: 36px;
    }
}
h3 {
    font-weight: 400;
    margin: 24px 0;
    font-size: 20px;
    @media (min-width: @screen-sm-min) {
        font-size: 22px;
    }
    @media (min-width: @screen-md-min) {
        font-size: 24px;
    }
}

code {
    font-size: 80%;
    color: #A83838;
    background-color: @grey-background-light;
}
pre {
    border-radius: 0;
    background-color: @grey-background-light;
    border-color: @background-colored;
}
pre code {
    background: transparent;
}
.hljs {
    background: transparent;
    padding: 0;
}

.btn {
    border-radius: 0;
}

.navbar {
    .navbar-default {
        background-color: @grey-background-light;
    }
    .navbar-nav {
        a {
            text-transform: uppercase;
            font-size: 11px;
            letter-spacing: .1rem;
            padding-left: 20px;
            padding-right: 20px;
        }
        a.github {
            padding-bottom: 10px;
            i {
                font-size: 20px;
            }
        }
    }
    #search-input {
        width: 180px;
        margin-left: 30px;
        background-color: #FBFBFB;
        border: 1px solid #E4E4E4;
        @media (min-width: @screen-lg-min) {
            width: 300px;
        }
    }
}

article {
    position: relative;
    padding: 30px 0 50px 0;
    h1 {
        margin: 0 0 32px;
        padding-bottom: 32px;
        border-bottom: solid 1px @border-color;
    }
    // Offset so that the title is not hidden behind the nav bar
    *[id]:before {
        display: block;
        content: " ";
        margin-top: -75px;
        height: 75px;
        visibility: hidden;
    }
    .anchor {
        position: absolute;
        margin-left: -25px;
        left: 0;
    }
    a.anchor:hover {
        text-decoration: none;
    }
    img {
        max-width: 100%;
    }
}

.documentation {
    .sidebar {
        text-align: right;
        float: left;
        padding: 35px 10px 35px 0;
        width: 120px;
        @media (min-width: @screen-sm-min) {
            width: 150px;
        }
        @media (min-width: @screen-md-min) {
            width: 200px;
            padding-right: 25px;
        }
        @media (min-width: @screen-lg-min) {
            width: 250px;
            padding-right: 35px;
        }

        ul {
            list-style: none;
            padding: 0;
            li {
                font-weight: 400;
                margin-bottom: 20px;
                font-size: 16px;
                @media (min-width: @screen-md-min) {
                    font-size: 18px;
                }
                ul {
                    display: block;
                    list-style: none;
                    margin: 10px 0 0 0;
                    padding: 0 0 0 10px;
                    li {
                        margin-bottom: 5px;
                        font-size: 12px;
                        @media (min-width: @screen-md-min) {
                            font-size: 14px;
                        }
                        &.active a {
                            color: @link-hover-color;
                            text-decoration: underline;
                        }
                    }
                }
            }
        }
    }
    article {
        margin-left: 130px;
        @media (min-width: @screen-sm-min) {
            margin-left: 200px;
        }
        @media (min-width: @screen-md-min) {
            margin-left: 250px;
        }
        @media (min-width: @screen-lg-min) {
            margin-left: 300px;
        }
    }
}

footer {
    background: @background-colored;
    margin-top: 50px;
    padding: 40px 0;
    font-size: 90%;
    text-align: center;
    color: #9e9e9e;
}

.gitter-chat-embed {
    z-index: 2000;
}
