// Bootstrap variables
@brand-primary: #D5A51F;
@text-color: #4A4A4A;

@grey-background-light: #f8f8f8;
@background-colored: #e7e7e7;
@border-color: #f0f2f1;


@link-color: darken(@brand-primary, 5%);
@link-hover-color: #9D5A08;
